@@ -27,12 +27,12 @@
 #include "config.h"
 #include <math.h>
 #include <pango/pangocairo.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include "gdkcairo.h"
 #include "gdkdrawable.h"
 #include "gdkinternals.h"
 #include "gdkwindow.h"
 #include "gdkscreen.h"
-#include "gdk-pixbuf-private.h"
 #include "gdkpixbuf.h"
 #include "gdkalias.h"
 
@@ -99,7 +99,7 @@ gdk_drawable_init (GdkDrawable *drawable)
  * @drawable: a #GdkDrawable
  * @key: name to store the data under
  * @data: arbitrary data
- * @destroy_func: function to free @data, or %NULL
+ * @destroy_func: (allow-none): function to free @data, or %NULL
  *
  * This function is equivalent to g_object_set_data(),
  * the #GObject variant should be used instead.
@@ -142,8 +142,8 @@ gdk_drawable_get_data (GdkDrawable   *drawable,
 /**
  * gdk_drawable_get_size:
  * @drawable: a #GdkDrawable
- * @width: (out): location to store drawable's width, or %NULL
- * @height: (out): location to store drawable's height, or %NULL
+ * @width: (out) (allow-none): location to store drawable's width, or %NULL
+ * @height: (out) (allow-none): location to store drawable's height, or %NULL
  *
  * Fills *@width and *@height with the size of @drawable.
  * @width or @height can be %NULL if you only want the other one.
@@ -151,8 +151,9 @@ gdk_drawable_get_data (GdkDrawable   *drawable,
  * On the X11 platform, if @drawable is a #GdkWindow, the returned
  * size is the size reported in the most-recently-processed configure
  * event, rather than the current size on the X server.
- * 
- **/
+ *
+ * Deprecated: 2.24: Use gdk_window_get_width() and gdk_window_get_height()
+ */
 void
 gdk_drawable_get_size (GdkDrawable *drawable,
 		       gint        *width,
@@ -160,8 +161,7 @@ gdk_drawable_get_size (GdkDrawable *drawable,
 {
   g_return_if_fail (GDK_IS_DRAWABLE (drawable));
 
-  if (GDK_DRAWABLE_GET_CLASS (drawable)->get_size != NULL)
-    GDK_DRAWABLE_GET_CLASS (drawable)->get_size (drawable, width, height);  
+  GDK_DRAWABLE_GET_CLASS (drawable)->get_size (drawable, width, height);  
 }
 
 /**
@@ -171,7 +171,9 @@ gdk_drawable_get_size (GdkDrawable *drawable,
  * Gets the #GdkVisual describing the pixel format of @drawable.
  * 
  * Return value: a #GdkVisual
- **/
+ *
+ * Deprecated: 2.24: Use gdk_window_get_visual()
+ */
 GdkVisual*
 gdk_drawable_get_visual (GdkDrawable *drawable)
 {
@@ -206,9 +208,11 @@ gdk_drawable_get_depth (GdkDrawable *drawable)
  * Return value: the #GdkScreen associated with @drawable
  *
  * Since: 2.2
+ *
+ * Deprecated: 2.24: Use gdk_window_get_screen() instead
  **/
 GdkScreen*
-gdk_drawable_get_screen(GdkDrawable *drawable)
+gdk_drawable_get_screen (GdkDrawable *drawable)
 {
   g_return_val_if_fail (GDK_IS_DRAWABLE (drawable), NULL);
 
@@ -224,15 +228,17 @@ gdk_drawable_get_screen(GdkDrawable *drawable)
  * Return value: the #GdkDisplay associated with @drawable
  *
  * Since: 2.2
+ *
+ * Deprecated: 2.24: Use gdk_window_get_display() instead
  **/
 GdkDisplay*
 gdk_drawable_get_display (GdkDrawable *drawable)
 {
   g_return_val_if_fail (GDK_IS_DRAWABLE (drawable), NULL);
-  
+
   return gdk_screen_get_display (gdk_drawable_get_screen (drawable));
 }
-	
+
 /**
  * gdk_drawable_set_colormap:
  * @drawable: a #GdkDrawable
@@ -321,6 +327,9 @@ gdk_drawable_unref (GdkDrawable *drawable)
  * 
  * Draws a point, using the foreground color and other attributes of 
  * the #GdkGC.
+ *
+ * Deprecated: 2.22: Use cairo_rectangle() and cairo_fill() or 
+ * cairo_move_to() and cairo_stroke() instead.
  **/
 void
 gdk_draw_point (GdkDrawable *drawable,
@@ -350,6 +359,18 @@ gdk_draw_point (GdkDrawable *drawable,
  * 
  * Draws a line, using the foreground color and other attributes of 
  * the #GdkGC.
+ *
+ * Deprecated: 2.22: Use cairo_line_to() and cairo_stroke() instead.
+ * Be aware that the default line width in Cairo is 2 pixels and that your
+ * coordinates need to describe the center of the line. To draw a single
+ * pixel wide pixel-aligned line, you would use:
+ * |[cairo_set_line_width (cr, 1.0);
+ * cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
+ * cairo_move_to (cr, 0.5, 0.5);
+ * cairo_line_to (cr, 9.5, 0.5);
+ * cairo_stroke (cr);]|
+ * See also <ulink url="http://cairographics.org/FAQ/#sharp_lines">the Cairo
+ * FAQ</ulink> on this topic.
  **/
 void
 gdk_draw_line (GdkDrawable *drawable,
@@ -391,6 +412,10 @@ gdk_draw_line (GdkDrawable *drawable,
  * <literal>gdk_draw_rectangle (window, gc, FALSE, 0, 0, 20, 20)</literal> 
  * results in an outlined rectangle with corners at (0, 0), (0, 20), (20, 20),
  * and (20, 0), which makes it 21 pixels wide and 21 pixels high.
+ *
+ * Deprecated: 2.22: Use cairo_rectangle() and cairo_fill() or cairo_stroke()
+ * instead. For stroking, the same caveats for converting code apply as for
+ * gdk_draw_line().
  **/
 void
 gdk_draw_rectangle (GdkDrawable *drawable,
@@ -438,6 +463,10 @@ gdk_draw_rectangle (GdkDrawable *drawable,
  * Draws an arc or a filled 'pie slice'. The arc is defined by the bounding
  * rectangle of the entire ellipse, and the start and end angles of the part 
  * of the ellipse to be drawn.
+ *
+ * Deprecated: 2.22: Use cairo_arc() and cairo_fill() or cairo_stroke()
+ * instead. Note that arcs just like any drawing operation in Cairo are
+ * antialiased unless you call cairo_set_antialias().
  **/
 void
 gdk_draw_arc (GdkDrawable *drawable,
@@ -482,6 +511,9 @@ gdk_draw_arc (GdkDrawable *drawable,
  * @n_points: the number of points.
  * 
  * Draws an outlined or filled polygon.
+ *
+ * Deprecated: 2.22: Use cairo_line_to() or cairo_append_path() and
+ * cairo_fill() or cairo_stroke() instead.
  **/
 void
 gdk_draw_polygon (GdkDrawable    *drawable,
@@ -625,6 +657,10 @@ gdk_draw_text_wc (GdkDrawable	 *drawable,
  * a color drawable. The way to draw a bitmap is to set the bitmap as 
  * the stipple on the #GdkGC, set the fill mode to %GDK_STIPPLED, and 
  * then draw the rectangle.
+ *
+ * Deprecated: 2.22: Use gdk_cairo_set_source_pixmap(), cairo_rectangle()
+ * and cairo_fill() to draw pixmap on top of other drawables. Also keep
+ * in mind that the limitations on allowed sources do not apply to Cairo.
  **/
 void
 gdk_draw_drawable (GdkDrawable *drawable,
@@ -705,6 +741,9 @@ gdk_draw_drawable (GdkDrawable *drawable,
  * 
  * Draws a #GdkImage onto a drawable.
  * The depth of the #GdkImage must match the depth of the #GdkDrawable.
+ *
+ * Deprecated: 2.22: Do not use #GdkImage anymore, instead use Cairo image
+ * surfaces.
  **/
 void
 gdk_draw_image (GdkDrawable *drawable,
@@ -733,7 +772,7 @@ gdk_draw_image (GdkDrawable *drawable,
 /**
  * gdk_draw_pixbuf:
  * @drawable: Destination drawable.
- * @gc: a #GdkGC, used for clipping, or %NULL
+ * @gc: (allow-none): a #GdkGC, used for clipping, or %NULL
  * @pixbuf: a #GdkPixbuf
  * @src_x: Source X coordinate within pixbuf.
  * @src_y: Source Y coordinates within pixbuf.
@@ -761,6 +800,9 @@ gdk_draw_image (GdkDrawable *drawable,
  * variable.
  *
  * Since: 2.2
+ *
+ * Deprecated: 2.22: Use gdk_cairo_set_source_pixbuf() and cairo_paint() or
+ * cairo_rectangle() and cairo_fill() instead.
  **/
 void
 gdk_draw_pixbuf (GdkDrawable     *drawable,
@@ -804,6 +846,9 @@ gdk_draw_pixbuf (GdkDrawable     *drawable,
  * 
  * Draws a number of points, using the foreground color and other 
  * attributes of the #GdkGC.
+ *
+ * Deprecated: 2.22: Use @n_points calls to cairo_rectangle() and
+ * cairo_fill() instead.
  **/
 void
 gdk_draw_points (GdkDrawable    *drawable,
@@ -833,6 +878,10 @@ gdk_draw_points (GdkDrawable    *drawable,
  *   @segs array.
  * 
  * Draws a number of unconnected lines.
+ *
+ * Deprecated: 2.22: Use cairo_move_to(), cairo_line_to() and cairo_stroke()
+ * instead. See the documentation of gdk_draw_line() for notes on line drawing
+ * with Cairo.
  **/
 void
 gdk_draw_segments (GdkDrawable      *drawable,
@@ -864,6 +913,9 @@ gdk_draw_segments (GdkDrawable      *drawable,
  * The way in which joins between lines are draw is determined by the
  * #GdkCapStyle value in the #GdkGC. This can be set with
  * gdk_gc_set_line_attributes().
+ *
+ * Deprecated: 2.22: Use cairo_line_to() and cairo_stroke() instead. See the
+ * documentation of gdk_draw_line() for notes on line drawing with Cairo.
  **/
 void
 gdk_draw_lines (GdkDrawable    *drawable,
@@ -935,6 +987,7 @@ real_draw_glyphs (GdkDrawable       *drawable,
  * understand; thus, use gdk_draw_layout() instead of this function,
  * gdk_draw_layout() handles the details.
  * 
+ * Deprecated: 2.22: Use pango_cairo_show_glyphs() instead.
  **/
 void
 gdk_draw_glyphs (GdkDrawable      *drawable,
@@ -955,7 +1008,7 @@ gdk_draw_glyphs (GdkDrawable      *drawable,
  * gdk_draw_glyphs_transformed:
  * @drawable: a #GdkDrawable
  * @gc: a #GdkGC
- * @matrix: a #PangoMatrix, or %NULL to use an identity transformation
+ * @matrix: (allow-none): a #PangoMatrix, or %NULL to use an identity transformation
  * @font: the font in which to draw the string
  * @x:       the x position of the start of the string (in Pango
  *           units in user space coordinates)
@@ -973,6 +1026,8 @@ gdk_draw_glyphs (GdkDrawable      *drawable,
  * See also gdk_draw_glyphs(), gdk_draw_layout().
  *
  * Since: 2.6
+ * 
+ * Deprecated: 2.22: Use pango_cairo_show_glyphs() instead.
  **/
 void
 gdk_draw_glyphs_transformed (GdkDrawable       *drawable,
@@ -1004,6 +1059,9 @@ gdk_draw_glyphs_transformed (GdkDrawable       *drawable,
  * likely not useful for applications.
  *
  * Since: 2.6
+ *
+ * Deprecated: 2.22: Use Cairo path contruction functions and cairo_fill()
+ * instead.
  **/
 void
 gdk_draw_trapezoids (GdkDrawable        *drawable,
@@ -1038,7 +1096,7 @@ gdk_draw_trapezoids (GdkDrawable        *drawable,
 /**
  * gdk_drawable_copy_to_image:
  * @drawable: a #GdkDrawable
- * @image: a #GdkDrawable, or %NULL if a new @image should be created.
+ * @image: (allow-none): a #GdkDrawable, or %NULL if a new @image should be created.
  * @src_x: x coordinate on @drawable
  * @src_y: y coordinate on @drawable
  * @dest_x: x coordinate within @image. Must be 0 if @image is %NULL
@@ -1054,6 +1112,9 @@ gdk_draw_trapezoids (GdkDrawable        *drawable,
  *               of @drawable
  * 
  * Since: 2.4
+ *
+ * Deprecated: 2.22: Use @drawable as the source and draw to a Cairo image
+ * surface if you want to download contents to the client.
  **/
 GdkImage*
 gdk_drawable_copy_to_image (GdkDrawable *drawable,
@@ -1150,6 +1211,9 @@ gdk_drawable_copy_to_image (GdkDrawable *drawable,
  * will contain undefined data.
  * 
  * Return value: a #GdkImage containing the contents of @drawable
+ *
+ * Deprecated: 2.22: Use @drawable as the source and draw to a Cairo image
+ * surface if you want to download contents to the client.
  **/
 GdkImage*
 gdk_drawable_get_image (GdkDrawable *drawable,
@@ -1555,20 +1619,21 @@ gdk_drawable_real_draw_pixbuf (GdkDrawable  *drawable,
   GdkDrawable  *real_drawable;
 
   g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
-  g_return_if_fail (pixbuf->colorspace == GDK_COLORSPACE_RGB);
-  g_return_if_fail (pixbuf->n_channels == 3 || pixbuf->n_channels == 4);
-  g_return_if_fail (pixbuf->bits_per_sample == 8);
+  g_return_if_fail (gdk_pixbuf_get_colorspace (pixbuf) == GDK_COLORSPACE_RGB);
+  g_return_if_fail (gdk_pixbuf_get_n_channels (pixbuf) == 3 ||
+                    gdk_pixbuf_get_n_channels (pixbuf) == 4);
+  g_return_if_fail (gdk_pixbuf_get_bits_per_sample (pixbuf) == 8);
 
   g_return_if_fail (drawable != NULL);
 
   if (width == -1) 
-    width = pixbuf->width;
+    width = gdk_pixbuf_get_width (pixbuf);
   if (height == -1)
-    height = pixbuf->height;
+    height = gdk_pixbuf_get_height (pixbuf);
 
   g_return_if_fail (width >= 0 && height >= 0);
-  g_return_if_fail (src_x >= 0 && src_x + width <= pixbuf->width);
-  g_return_if_fail (src_y >= 0 && src_y + height <= pixbuf->height);
+  g_return_if_fail (src_x >= 0 && src_x + width <= gdk_pixbuf_get_width (pixbuf));
+  g_return_if_fail (src_y >= 0 && src_y + height <= gdk_pixbuf_get_height (pixbuf));
 
   /* Clip to the drawable; this is required for get_from_drawable() so
    * can't be done implicitly
@@ -1638,7 +1703,7 @@ gdk_drawable_real_draw_pixbuf (GdkDrawable  *drawable,
   else
     real_drawable = GDK_PIXMAP_OBJECT (drawable)->impl;
 
-  if (pixbuf->has_alpha)
+  if (gdk_pixbuf_get_has_alpha (pixbuf))
     {
       GdkVisual *visual = gdk_drawable_get_visual (drawable);
       void (*composite_func) (guchar       *src_buf,
@@ -1701,8 +1766,8 @@ gdk_drawable_real_draw_pixbuf (GdkDrawable  *drawable,
 					      dest_x + x0, dest_y + y0,
 					      xs0, ys0,
 					      width1, height1);
-		  (*composite_func) (pixbuf->pixels + (src_y + y0) * pixbuf->rowstride + (src_x + x0) * 4,
-				     pixbuf->rowstride,
+		  (*composite_func) (gdk_pixbuf_get_pixels (pixbuf) + (src_y + y0) * gdk_pixbuf_get_rowstride (pixbuf) + (src_x + x0) * 4,
+				     gdk_pixbuf_get_rowstride (pixbuf),
 				     (guchar*)image->mem + ys0 * image->bpl + xs0 * image->bpp,
 				     image->bpl,
 				     visual->byte_order,
@@ -1729,10 +1794,10 @@ gdk_drawable_real_draw_pixbuf (GdkDrawable  *drawable,
 						     width, height);
 	  
 	  if (composited)
-	    composite (pixbuf->pixels + src_y * pixbuf->rowstride + src_x * 4,
-		       pixbuf->rowstride,
-		       composited->pixels,
-		       composited->rowstride,
+	    composite (gdk_pixbuf_get_pixels (pixbuf) + src_y * gdk_pixbuf_get_rowstride (pixbuf) + src_x * 4,
+		       gdk_pixbuf_get_rowstride (pixbuf),
+		       gdk_pixbuf_get_pixels (composited),
+		       gdk_pixbuf_get_rowstride (composited),
 		       width, height);
 	}
     }
@@ -1744,26 +1809,26 @@ gdk_drawable_real_draw_pixbuf (GdkDrawable  *drawable,
       pixbuf = composited;
     }
   
-  if (pixbuf->n_channels == 4)
+  if (gdk_pixbuf_get_n_channels (pixbuf) == 4)
     {
-      guchar *buf = pixbuf->pixels + src_y * pixbuf->rowstride + src_x * 4;
+      guchar *buf = gdk_pixbuf_get_pixels (pixbuf) + src_y * gdk_pixbuf_get_rowstride (pixbuf) + src_x * 4;
 
       gdk_draw_rgb_32_image_dithalign (real_drawable, gc,
 				       dest_x, dest_y,
 				       width, height,
 				       dither,
-				       buf, pixbuf->rowstride,
+				       buf, gdk_pixbuf_get_rowstride (pixbuf),
 				       x_dither, y_dither);
     }
   else				/* n_channels == 3 */
     {
-      guchar *buf = pixbuf->pixels + src_y * pixbuf->rowstride + src_x * 3;
+      guchar *buf = gdk_pixbuf_get_pixels (pixbuf) + src_y * gdk_pixbuf_get_rowstride (pixbuf) + src_x * 3;
 
       gdk_draw_rgb_image_dithalign (real_drawable, gc,
 				    dest_x, dest_y,
 				    width, height,
 				    dither,
-				    buf, pixbuf->rowstride,
+				    buf, gdk_pixbuf_get_rowstride (pixbuf),
 				    x_dither, y_dither);
     }
 
